/* GAdmin-Antivirus - An easy to use GTK+ frontend for clamav antivirus.
 * Copyright (C) 2010 - 2014 Magnus Loef <magnus-swe@telia.com> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
*/



#include "../config.h"
#include <gtk/gtk.h>
#include "gettext.h"
#include "support.h"
#include "widgets.h"
#include "credits_window.h"
#include "activate_scan.h"
#include "save_scan_settings.h"
#include "show_help.h"


void
create_main_window(struct w *widgets)
{
    gchar *info, *pixmap_directory;
    GtkCellRenderer *pixbuf_cell_renderer;
    GdkPixbuf *pixbuf;

    /* Create the main window */
    widgets->main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position(GTK_WINDOW(widgets->main_window),
                                           GTK_WIN_POS_CENTER);
    gtk_widget_set_size_request(widgets->main_window, -1, 450);

    /* Set window information */
    info = g_strdup_printf("GADMIN-ANTIVIRUS %s", VERSION);
    gtk_window_set_title(GTK_WINDOW(widgets->main_window), info);
    g_free(info);

    /* Set the main window icon */
    pixmap_directory =
        g_strdup_printf("%s/pixmaps/gadmin-antivirus", PACKAGE_DATA_DIR);
    add_pixmap_directory(pixmap_directory);
    g_free(pixmap_directory);

    pixbuf_cell_renderer = gtk_cell_renderer_pixbuf_new();
    pixbuf = create_pixbuf("gadmin-antivirus.png");
    g_object_set(pixbuf_cell_renderer, "pixbuf", pixbuf, NULL);
    gtk_window_set_icon(GTK_WINDOW(widgets->main_window), pixbuf);
    g_object_unref(pixbuf);

    widgets->main_vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
    gtk_container_add(GTK_CONTAINER(widgets->main_window), widgets->main_vbox);

    /* Hboxes (down) */
    GtkWidget *toolbar_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(widgets->main_vbox),
                      toolbar_hbox, FALSE, FALSE, 0);

    GtkWidget *status_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(widgets->main_vbox),
                       status_hbox, FALSE, FALSE, 0);

    GtkWidget *status_hsep_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(widgets->main_vbox),
                   status_hsep_hbox, FALSE, TRUE, 0);

    GtkWidget *notebook_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(widgets->main_vbox),
                       notebook_hbox, TRUE, TRUE, 0);

    /* Create the main toolbar */
    GtkWidget *main_toolbar = gtk_toolbar_new();
    gtk_box_pack_start(GTK_BOX(toolbar_hbox), main_toolbar, TRUE, TRUE, 0);
    gtk_toolbar_set_style(GTK_TOOLBAR(main_toolbar), GTK_TOOLBAR_BOTH);

    /* Activate button */
    GtkWidget *toolbar_icon_activate = gtk_image_new_from_icon_name("gtk-yes",
                                   gtk_toolbar_get_icon_size(GTK_TOOLBAR(main_toolbar)));
    GtkToolItem *activate_button = gtk_tool_button_new(GTK_WIDGET(toolbar_icon_activate), _("Activate"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar), activate_button, -1);
/*
    GtkToolItem *activate_button = gtk_tool_button_new_from_stock("gtk-yes");
    gtk_tool_button_set_label(GTK_TOOL_BUTTON(activate_button), _("Activate"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar),
                GTK_TOOL_ITEM(activate_button), 0);
*/
    g_signal_connect_swapped(G_OBJECT(activate_button), "clicked",
        G_CALLBACK(scan_button_clicked), widgets);

    /* Deactivate button */
    GtkWidget *toolbar_icon_deactivate = gtk_image_new_from_icon_name("gtk-no",
                                   gtk_toolbar_get_icon_size(GTK_TOOLBAR(main_toolbar)));
    GtkToolItem *deactivate_button = gtk_tool_button_new(GTK_WIDGET(toolbar_icon_deactivate), _("Deactivate"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar), deactivate_button, -1);
    g_signal_connect_swapped(G_OBJECT(deactivate_button), "clicked",
        G_CALLBACK(cancel_process_clicked), widgets);

    /* Help button */
    GtkWidget *toolbar_icon_help = gtk_image_new_from_icon_name("gtk-help",
                                   gtk_toolbar_get_icon_size(GTK_TOOLBAR(main_toolbar)));
    GtkToolItem *help_button = gtk_tool_button_new(GTK_WIDGET(toolbar_icon_help), _("Help"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar), help_button, -1);
    g_signal_connect_swapped(G_OBJECT(help_button), "clicked",
        G_CALLBACK(show_help), widgets);

    /* About button */
    GtkWidget *toolbar_icon_about = gtk_image_new_from_icon_name("gtk-about",
                                   gtk_toolbar_get_icon_size(GTK_TOOLBAR(main_toolbar)));
    GtkToolItem *about_button = gtk_tool_button_new(GTK_WIDGET(toolbar_icon_about), _("About"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar), about_button, -1);
    g_signal_connect_swapped(G_OBJECT(about_button), "clicked",
        G_CALLBACK(show_credits), widgets);

    /* Quit button */
    GtkWidget *toolbar_icon_quit = gtk_image_new_from_icon_name("gtk-quit",
                                   gtk_toolbar_get_icon_size(GTK_TOOLBAR(main_toolbar)));
    GtkToolItem *quit_button = gtk_tool_button_new(GTK_WIDGET(toolbar_icon_quit), _("Quit"));
    gtk_toolbar_insert(GTK_TOOLBAR(main_toolbar), quit_button, -1);
    g_signal_connect_swapped(G_OBJECT(quit_button), "clicked",
        G_CALLBACK(gtk_main_quit), widgets);

    /* The notebook */
    GtkWidget *notebook_vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
    gtk_box_pack_start(GTK_BOX(notebook_hbox), notebook_vbox, TRUE, TRUE, 0);

    widgets->notebook_vbox1 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
    widgets->notebook_vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);

    widgets->main_notebook = gtk_notebook_new();
    gtk_box_pack_start(GTK_BOX(notebook_vbox), widgets->main_notebook, TRUE, TRUE, 0);
    gtk_notebook_set_show_border(GTK_NOTEBOOK(widgets->main_notebook), TRUE);

    GtkWidget *settings_label = gtk_label_new(_("Antivirus settings"));

    GtkWidget *progress_label = gtk_label_new(_("Scan progress"));

    gtk_notebook_insert_page(GTK_NOTEBOOK(widgets->main_notebook),
        widgets->notebook_vbox1, settings_label, 0);
    gtk_notebook_insert_page(GTK_NOTEBOOK(widgets->main_notebook),
        widgets->notebook_vbox2, progress_label, 1);

    /* Set version and status labels */
    widgets->version_label = gtk_label_new(_("Information: cant read version"));
    gtk_box_pack_start(GTK_BOX(status_hbox), widgets->version_label, FALSE, FALSE, 0);
    gtk_misc_set_alignment(GTK_MISC(widgets->version_label), 0, 0);

    GtkWidget *status_spacer_label = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(status_hbox),
          status_spacer_label, TRUE, TRUE, 0);
    gtk_misc_set_alignment(GTK_MISC(status_spacer_label), 0, 0);

    widgets->status_label = gtk_label_new(_("Status: unknown"));
    gtk_box_pack_start(GTK_BOX(status_hbox),
      widgets->status_label, FALSE, FALSE, 0);
    gtk_misc_set_alignment(GTK_MISC(widgets->status_label), 0, 0);

    GtkWidget *status_hseparator = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
    gtk_box_pack_start(GTK_BOX(status_hsep_hbox),
                 status_hseparator, TRUE, TRUE, 0);
    gtk_widget_set_size_request(status_hseparator, 10, 10);

    gtk_widget_show_all(widgets->main_window);
}
