/* GAdmin-Antivirus - An easy to use GTK+ frontend for clamav antivirus.
 * Copyright (C) 2010 - 2014 Magnus Loef <magnus-swe@telia.com> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
*/



#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "widgets.h"
#include "gettext.h"
#include "create_settings_tab.h"
#include "populate_settings_tab.h"
#include "show_info.h"
#include "make_settings_checkbuttons.h"
#include "make_settings_spinbuttons.h"
#include "make_settings_entries.h"
#include "make_settings_combos.h"
#include "make_settings_labels.h"
#include "make_settings_hseparators.h"
#include "save_scan_settings.h"
#include "functions.h"
#include "cron_functions.h"
#include "scan_menu.h"
#include "show_log.h"

extern gchar *global_settings_dir;
extern gchar *global_scan_name;
/* Used to see if the scan combo changed */
extern gulong scan_combo_changed_signal;


/* Change cell text in a treeview */
void
cell_edited(GtkCellRendererText *cell,
            gchar *path_string, gchar *new_text, struct w *widgets)
{
    GtkTreeIter iter;
    GtkTreePath *path = NULL;

    int column = (int)g_object_get_data(G_OBJECT(cell), "column");

    path = gtk_tree_path_new_from_string(path_string);

    gtk_tree_model_get_iter(GTK_TREE_MODEL(widgets->scan_store), &iter, path);

    if(path != NULL)
        gtk_tree_path_free(path);

    gtk_list_store_set(GTK_LIST_STORE(widgets->scan_store), &iter, column,
        new_text, -1);
}


/* Change cell toggle value in a treeview */
void
treeview_toggle_button_clicked(GtkCellRendererToggle * cell_renderer,
    gchar * path, struct w *widgets)
{
    GtkTreePath *treepath;
    GtkTreeIter iter;
    gboolean val;
    treepath = gtk_tree_path_new_from_string(path);
    GtkTreeModel *model;
    int col_num;

    col_num = (int)g_object_get_data(G_OBJECT(cell_renderer), "column");

    gtk_tree_model_get_iter(GTK_TREE_MODEL(widgets->scan_store),
                                                 &iter, treepath);
    model = gtk_tree_view_get_model(GTK_TREE_VIEW(widgets->scan_treeview));

    /* Toggle it to the opposite checked state */
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, col_num, &val, -1);
    if(val)
        gtk_list_store_set(widgets->scan_store, &iter, col_num, FALSE, -1);
    else
        gtk_list_store_set(widgets->scan_store, &iter, col_num, TRUE, -1);

    gtk_tree_path_free(treepath);
}


/* Deletes the selected row from the treeview */
void
del_treeview_row(struct w *widgets)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreeIter iter;
    gboolean val = 0, edit = 0;
    GtkTreePath *path;

    selection =
        gtk_tree_view_get_selection(GTK_TREE_VIEW(widgets->scan_treeview));
    if(selection)
    {
        val =
            gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection),
            &model, &iter);

        if(gtk_list_store_iter_is_valid(GTK_LIST_STORE(model), &iter))
            gtk_list_store_remove(widgets->scan_store, &iter);

        path = gtk_tree_path_new_first();

        if(path)
            gtk_tree_view_set_cursor(GTK_TREE_VIEW(widgets->scan_treeview),
                                                           path, NULL, edit);
        gtk_tree_path_free(path);
    }

    save_scan_settings(widgets);
}


/* New scan name selected */
void
scan_name_combo_changed(struct w *widgets)
{
    int i = 0;

    /* Clear schedule days combo boxes */
    for(i = 0; i < 8; i++)
        gtk_toggle_button_set_active(
                  GTK_TOGGLE_BUTTON(widgets->schedule_check_button[i]), FALSE);

    /* Clear schedule time combo box */
    gtk_toggle_button_set_active(
                  GTK_TOGGLE_BUTTON(widgets->schedule_check_button[8]), FALSE);

    /* Set schedule spinbuttons to default time */
    for(i = 0; i < 2; i++)
        gtk_spin_button_set_value(
                    GTK_SPIN_BUTTON(widgets->schedule_spin_button[i]), 1);

    populate_settings_tab(widgets);
}


/* Creates a new scan (settings file) and selects it.
   Then shows the file selector */
void
create_new_scan(struct w *widgets)
{
    FILE *fp;
    int i = 0;
    gchar *info, *utf8, *conf_file;

    G_CONST_RETURN gchar *scan_name =
        gtk_entry_get_text(GTK_ENTRY(widgets->new_scan_entry));
    if(scan_name == NULL || strlen(scan_name) < 3)
    {
        info = g_strdup_printf(_("The scan name is too short.\n"));
        show_info(info);
        g_free(info);
        return;
    }

    /* Fixme: Some chars cannot be used in the scan name. */
    for(i = 0; scan_name[i] != '\0'; i++)
    {
        if(scan_name[i] == '/' || scan_name[i] == '\r'
            || scan_name[i] == '\n' || scan_name[i] == '\t')
        {
            info = g_strdup_printf(
             _("The scan name contains special chars that cannot be used.\n"));
            show_info(info);
            g_free(info);
            return;
        }
    }

    /* Create a new scan settings file */
    conf_file = g_strdup_printf("%s/%s", global_settings_dir, scan_name);

    /* If the scan exists, tell the user and return */
    if(file_exists(conf_file))
    {
        info = g_strdup_printf(_("A scan with this name already exists."));
        show_info(info);
        g_free(info);
        g_free(conf_file);
        return;
    }

    /* Write the new settings file */
    if((fp = fopen(conf_file, "w+")) == NULL)
    {
        info = g_strdup_printf(
            _("Error: Can not write scan settings here:\n%s\n"), conf_file);
        show_info(info);
        g_free(info);
        g_free(conf_file);
        return;
    }
    fclose(fp);
    g_free(conf_file);

    /* Switch to this scan name */
    global_scan_name = g_strdup_printf("%s", scan_name);

    /* Put this new scan name first in the combo and select it */
    utf8 = g_locale_to_utf8(scan_name, strlen(scan_name), NULL, NULL, NULL);
    gtk_combo_box_text_prepend(GTK_COMBO_BOX_TEXT(widgets->settings_combo[0]), NULL, utf8);
    gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[0]), 0);
    if(utf8 != NULL)
        g_free(utf8);

    populate_settings_tab(widgets);

    gtk_widget_destroy(widgets->new_scan_window);

    /* Show scan path fileselector */
    show_path_filesel(widgets);
}


/* Deletes the selected scan, schedule and logs then repopulates the gui */
void
delete_scan(struct w *widgets)
{
    gchar *info, *conf_file, *script_file, *log_file;

    conf_file = g_strdup_printf("%s/%s",
                                global_settings_dir, global_scan_name);

    if(global_scan_name == NULL || strlen(global_scan_name) < 1)
    {
        info = g_strdup_printf(_("Scan name too short, scan not deleted\n"));
        show_info(info);
        g_free(info);
        g_free(conf_file);
        return;
    }

    /* Delete the settings file */
    if(conf_file != NULL)
    {
        unlink(conf_file);
        g_free(conf_file);
    }

    /* Delete the schedule script if any */
    script_file = mk_scan_script_path(global_scan_name);
    if(script_file != NULL && strstr(script_file, "gadmin-antivirus"))
        unlink(script_file);

    /* Delete the cron line if any */
    if(script_file != NULL && strstr(script_file, "gadmin-antivirus"))
        del_cron(script_file);

    if(script_file != NULL)
        g_free(script_file);

    /* Delete the log file if any */
    log_file = mk_log_path(global_scan_name);
    if(log_file != NULL && strstr(log_file, "gadmin-antivirus"))
        unlink(log_file);

    if(log_file != NULL)
        g_free(log_file);

    /* Clear the "scan name" combo. Block its signal first. */
    g_signal_handler_block(GTK_WIDGET(widgets->settings_combo[0]),
        scan_combo_changed_signal);

    GtkTreeModel *model =
        gtk_combo_box_get_model(GTK_COMBO_BOX(widgets->settings_combo[0]));
    gtk_list_store_clear(GTK_LIST_STORE(model));

    /* Unblock its signal */
    g_signal_handler_unblock(GTK_WIDGET(widgets->settings_combo[0]),
        scan_combo_changed_signal);

    populate_settings_tab(widgets);
}


/* Shows a window with a scan name entry */
void
create_new_scan_window(struct w *widgets)
{
    GtkWidget *vbox;
    gchar *info;

    /* Create the new scan window */
    widgets->new_scan_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position(GTK_WINDOW(widgets->new_scan_window),
        GTK_WIN_POS_CENTER);
    gtk_widget_set_size_request(widgets->new_scan_window, 600, 200);

    /* Set window information */
    info = g_strdup_printf("Gadmin-Antivirus %s - Create new scan", VERSION);
    gtk_window_set_title(GTK_WINDOW(widgets->new_scan_window), info);
    g_free(info);

    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
    gtk_container_add(GTK_CONTAINER(widgets->new_scan_window), vbox);

    /* scan name entry inside a table and frame */
    GtkWidget *frame = gtk_frame_new(_("New scan settings:"));
    GtkWidget *grid = gtk_grid_new();
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 1);
    gtk_container_add(GTK_CONTAINER(frame), grid);

    widgets->new_scan_entry =
        make_entry_with_label(GTK_GRID(grid),
        _(" The name of the new scan: "), 0, 1, 0, 1, 300);

    /* Apply and cancel buttons */
    GtkWidget *hbuttonbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
    gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 0);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_SPREAD);
    GtkWidget *cancel_button = gtk_button_new_from_icon_name("gtk-cancel", GTK_ICON_SIZE_BUTTON);
    gtk_container_add(GTK_CONTAINER(hbuttonbox), cancel_button);
    g_signal_connect_swapped(G_OBJECT(cancel_button), "clicked",
        G_CALLBACK(gtk_widget_destroy), G_OBJECT(widgets->new_scan_window));

    GtkWidget *apply_button = gtk_button_new_from_icon_name("gtk-apply", GTK_ICON_SIZE_BUTTON);
    gtk_container_add(GTK_CONTAINER(hbuttonbox), apply_button);
    g_signal_connect_swapped(G_OBJECT(apply_button), "clicked",
        G_CALLBACK(create_new_scan), widgets);

    gtk_widget_show_all(widgets->new_scan_window);
}


void
create_settings_tab(struct w *widgets)
{
    GtkWidget *settings_treeview_hbox;
    GtkWidget *directory_scrolled_window;
    GtkCellRenderer *path_cell_renderer;
    GtkCellRenderer *include_cell_renderer, *exclude_cell_renderer;
    GtkCellRenderer *incl_scan_cell_renderer;
    GtkTreeViewColumn *col[11];
    int colnr = 0;
    int acol = 0, arow = 1;
    GtkWidget *frame;
    gchar *utf8 = NULL;

    /* Create the scrolled window with a frame and a table */
    settings_treeview_hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(widgets->notebook_vbox1),
        settings_treeview_hbox, TRUE, TRUE, 0);

    widgets->settings_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(settings_treeview_hbox),
        widgets->settings_scrolled_window, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
        (widgets->settings_scrolled_window),
        GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
    /* Must set a larger size or it wont scroll */
    gtk_widget_set_size_request(widgets->settings_scrolled_window, -1, 100);

    /* The main vbox */
    widgets->settings_vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
    gtk_container_add(GTK_CONTAINER(widgets->settings_scrolled_window), widgets->settings_vbox);

    /* Create the directory treeview in the user settings scrolled window */
    directory_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(widgets->settings_vbox),
        directory_scrolled_window, FALSE, FALSE, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
        (directory_scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* Must set a larger size or it wont scroll */
    gtk_widget_set_size_request(directory_scrolled_window, -1, 120);

    /* 4 columns */
    widgets->scan_store =
        gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
        G_TYPE_BOOLEAN);

    widgets->scan_treeview = gtk_tree_view_new();
    gtk_tree_view_set_model(GTK_TREE_VIEW(widgets->scan_treeview),
        GTK_TREE_MODEL(widgets->scan_store));

    gtk_container_add(GTK_CONTAINER(directory_scrolled_window),
        widgets->scan_treeview);
    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(widgets->scan_treeview), TRUE);


    /* A hbuttonbox below the treeview */
    GtkWidget *hbuttonbox1;
    hbuttonbox1 = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
    gtk_box_pack_start(GTK_BOX(widgets->settings_vbox),
                            hbuttonbox1, FALSE, FALSE, 0);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox1),
                                      GTK_BUTTONBOX_SPREAD);

    /* Add scan data button */
    GtkWidget *add_scan_button = gtk_button_new();
    gtk_widget_set_tooltip_text(add_scan_button, _("Add data to this scan."));
    GtkWidget *alignment62 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(add_scan_button), alignment62);
    GtkWidget *hbox60 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_container_add(GTK_CONTAINER(alignment62), hbox60);
    GtkWidget *image52 = gtk_image_new_from_icon_name("gtk-add",
                                        GTK_ICON_SIZE_BUTTON);

    gtk_box_pack_start(GTK_BOX(hbox60), image52, FALSE, FALSE, 0);
    GtkWidget *label230 = gtk_label_new_with_mnemonic(_("Add scan data"));
    gtk_box_pack_start(GTK_BOX(hbox60), label230, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label230), GTK_JUSTIFY_LEFT);
    /* The add file or directory signal hookup */
    g_signal_connect_swapped(add_scan_button, "clicked",
        G_CALLBACK(show_path_filesel), widgets);
    gtk_box_pack_start(GTK_BOX(hbuttonbox1), add_scan_button, TRUE, TRUE, 0);


    /* Remove data selection button */
    GtkWidget *del_scan_row_button = gtk_button_new();
    gtk_widget_set_tooltip_text(del_scan_row_button,
        _("Remove selection from scan."));
    GtkWidget *alignment64 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(del_scan_row_button), alignment64);
    GtkWidget *hbox64 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_container_add(GTK_CONTAINER(alignment64), hbox64);
    GtkWidget *image56 = gtk_image_new_from_icon_name("gtk-delete", GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start(GTK_BOX(hbox64), image56, FALSE, FALSE, 0);
    GtkWidget *label234 = gtk_label_new_with_mnemonic(_("Remove selected data"));
    gtk_box_pack_start(GTK_BOX(hbox64), label234, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label234), GTK_JUSTIFY_LEFT);
    /* Remove data selection signal hookup */
    g_signal_connect_swapped(del_scan_row_button, "clicked",
                       G_CALLBACK(del_treeview_row), widgets);
    gtk_box_pack_start(GTK_BOX(hbuttonbox1),
          del_scan_row_button, TRUE, TRUE, 0);

    /* Setup the treeview cells to be editable or clickable */
    path_cell_renderer = gtk_cell_renderer_text_new();
    g_object_set(path_cell_renderer, "editable", TRUE, NULL);
    g_signal_connect(path_cell_renderer, "edited",
                  G_CALLBACK(cell_edited), widgets);

    include_cell_renderer = gtk_cell_renderer_text_new();
    g_object_set(include_cell_renderer, "editable", TRUE, NULL);
    g_signal_connect(include_cell_renderer, "edited",
                     G_CALLBACK(cell_edited), widgets);

    exclude_cell_renderer = gtk_cell_renderer_text_new();
    g_object_set(exclude_cell_renderer, "editable", TRUE, NULL);
    g_signal_connect(exclude_cell_renderer, "edited",
                     G_CALLBACK(cell_edited), widgets);

    incl_scan_cell_renderer = gtk_cell_renderer_toggle_new();
    g_object_set(incl_scan_cell_renderer, "activatable", TRUE, NULL);
    g_signal_connect(incl_scan_cell_renderer, "toggled",
        G_CALLBACK(treeview_toggle_button_clicked), widgets);

    colnr = 0;

    col[colnr] = gtk_tree_view_column_new_with_attributes(_("Scan path"),
                                  path_cell_renderer, "text", colnr, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(widgets->scan_treeview),
                                      GTK_TREE_VIEW_COLUMN(col[colnr]));
    colnr++;

    col[colnr] = gtk_tree_view_column_new_with_attributes(_("Include pattern"),
                                     include_cell_renderer, "text", colnr, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(widgets->scan_treeview),
                                      GTK_TREE_VIEW_COLUMN(col[colnr]));
    colnr++;

    col[colnr] = gtk_tree_view_column_new_with_attributes(_("Exclude pattern"),
                                     exclude_cell_renderer, "text", colnr, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(widgets->scan_treeview),
                                      GTK_TREE_VIEW_COLUMN(col[colnr]));
    colnr++;

    col[colnr] = gtk_tree_view_column_new_with_attributes(_("Include in scan"),
                                 incl_scan_cell_renderer, "active", colnr, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(widgets->scan_treeview),
                                      GTK_TREE_VIEW_COLUMN(col[colnr]));
    colnr++;

    g_object_set_data(G_OBJECT(path_cell_renderer), "column", (void *)(0));
    g_object_set_data(G_OBJECT(include_cell_renderer), "column", (void *)(1));
    g_object_set_data(G_OBJECT(exclude_cell_renderer), "column", (void *)(2));
    g_object_set_data(G_OBJECT(incl_scan_cell_renderer), "column", (void *)(3));

    /* Add a settings frame to a vbox */
    frame = gtk_frame_new("");
    /* Add a grid to the frame in the vbox */
    widgets->settings_grid = gtk_grid_new();
    gtk_box_pack_start(GTK_BOX(widgets->settings_vbox), frame, TRUE, TRUE, 1);
    gtk_container_add(GTK_CONTAINER(frame), widgets->settings_grid);

    /* Spacer label */
    make_3columns_label(GTK_GRID(widgets->settings_grid), "", 2, arow, 1, 1);
    arow++;

    /* The settings... */

    /* "Delete selected scan" button */
    GtkWidget *del_scan_button = gtk_button_new();
    gtk_widget_set_tooltip_text(del_scan_button,
                                _("Delete the selected scan."));
    GtkWidget *alignment40 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(del_scan_button), alignment40);
    GtkWidget *hbox40 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
    gtk_container_add(GTK_CONTAINER(alignment40), hbox40);
    GtkWidget *image21 =
        gtk_image_new_from_icon_name("gtk-delete", GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start(GTK_BOX(hbox40), image21, FALSE, FALSE, 0);
    GtkWidget *label221 = gtk_label_new_with_mnemonic(
                                 _("Delete selected scan"));
    gtk_box_pack_start(GTK_BOX(hbox40), label221, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label221), GTK_JUSTIFY_LEFT);
    /* Delete selected scan signal hookup */
    g_signal_connect_swapped(del_scan_button, "clicked",
                        G_CALLBACK(delete_scan), widgets);

    /* Selected scan combo */
    utf8 = g_locale_to_utf8(_(" Selected scan: "), -1, NULL, NULL, NULL);
    widgets->settings_combo[0] =
        make_label_combo_button(GTK_GRID(widgets->settings_grid),
                             del_scan_button, utf8, acol, arow, 1, 1, 350);

    /* Callback for combo changed */
    scan_combo_changed_signal =
        g_signal_connect_swapped(widgets->settings_combo[0], "changed",
                           G_CALLBACK(scan_name_combo_changed), widgets);
    arow++;

    /* Spacer label */
    make_3columns_label(GTK_GRID(widgets->settings_grid), "", acol, arow, 1, 1);
    arow++;

    GtkWidget *hbuttonbox2;
    hbuttonbox2 = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
    gtk_grid_attach(GTK_GRID(widgets->settings_grid), hbuttonbox2, acol, arow, 3, 1);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox2),
                                      GTK_BUTTONBOX_SPREAD);

    /* "Create new scan" button */
    GtkWidget *create_new_scan_button = gtk_button_new();
    gtk_widget_set_tooltip_text(create_new_scan_button,
                                    _("Create a new scan."));
    GtkWidget *alignment52 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(create_new_scan_button), alignment52);

    GtkWidget *hbox50 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_container_add(GTK_CONTAINER(alignment52), hbox50);
    GtkWidget *image51 = gtk_image_new_from_icon_name("gtk-add",
                                        GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start(GTK_BOX(hbox50), image51, FALSE, FALSE, 0);

    GtkWidget *label220 = gtk_label_new_with_mnemonic(_("Create new scan"));
    gtk_box_pack_start(GTK_BOX(hbox50), label220, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label220), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(hbuttonbox2), create_new_scan_button, FALSE, FALSE, 0);
    /* The create new scan signal hookup */
    g_signal_connect_swapped(create_new_scan_button, "clicked",
                    G_CALLBACK(create_new_scan_window), widgets);

    /* Save scan button */
    GtkWidget *save_scan_button = gtk_button_new();
    gtk_widget_set_tooltip_text(save_scan_button, _("Save this scan."));
    GtkWidget *alignment68 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(save_scan_button), alignment68);
    GtkWidget *hbox68 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
    gtk_container_add(GTK_CONTAINER(alignment68), hbox68);
    GtkWidget *image58 =
        gtk_image_new_from_icon_name("gtk-refresh", GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start(GTK_BOX(hbox68), image58, FALSE, FALSE, 0);

    GtkWidget *label238 = gtk_label_new_with_mnemonic(_("Save scan"));
    gtk_box_pack_start(GTK_BOX(hbox68), label238, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label238), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(hbuttonbox2),
                            save_scan_button, FALSE, FALSE, 0);
    /* Save scan button signal hookup */
    g_signal_connect_swapped(save_scan_button, "clicked",
                  G_CALLBACK(save_scan_settings), widgets);
    arow++;

    /* Show log button */
    GtkWidget *log_scan_button = gtk_button_new();
    gtk_widget_set_tooltip_text(log_scan_button,
        _("Show log for the selected scan."));
    GtkWidget *alignment28 = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(log_scan_button), alignment28);
    GtkWidget *hbox28 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
    gtk_container_add(GTK_CONTAINER(alignment28), hbox28);
    GtkWidget *image28 = gtk_image_new_from_icon_name("gtk-find",
                                            GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start(GTK_BOX(hbox28), image28, FALSE, FALSE, 0);

    GtkWidget *label28 = gtk_label_new_with_mnemonic(_("Show scan log"));
    gtk_box_pack_start(GTK_BOX(hbox28), label28, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(label28), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(hbuttonbox2), log_scan_button, FALSE, FALSE, 0);
    /* Show log button signal hookup */
    g_signal_connect_swapped(log_scan_button, "clicked",
                    G_CALLBACK(show_log_window), widgets);
    arow++;

    /* Spacer label */
    make_3columns_label(GTK_GRID(widgets->settings_grid), "", acol, arow, 1, 1);
    arow++;

    /* CRON Time schedule settings */
    widgets->schedule_check_button[0] =
        make_3columns_label_spin(GTK_GRID(widgets->settings_grid),
        _("Schedule this scan to run at specific times: "), acol, arow, 1, 1);
    arow++;

    /* Hseparator */
    make_3columns_hseparator(GTK_GRID(widgets->settings_grid), acol, arow, 1, 1);
    arow++;

    /* Schedule (Month day = 1-31) */
    widgets->schedule_spin_button[0] =
        make_shortleft_spinbutton_with_label(
            GTK_GRID(widgets->settings_grid),
                      _("Month day"), acol, arow, 1, 1, 20, 1, 31);
    gtk_widget_set_tooltip_text(widgets->schedule_spin_button[0],
    _("The scan will run on the specified day of the month.\n"));
    arow++;

    /* Any month day (*) */
    widgets->schedule_check_button[1] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                         _("Not monthly: "), acol, arow, 1, 1);
    gtk_widget_set_tooltip_text(widgets->schedule_check_button[1],
                _("Month scheduling will be disabled if selected."));
    arow++;

    /* Hseparator */
    make_3columns_hseparator(GTK_GRID(widgets->settings_grid), acol, arow, 1, 1);
    arow++;

    /* Schedule days */
    widgets->schedule_check_button[2] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                            _("Monday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[3] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                           _("Tuesday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[4] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                         _("Wednesday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[5] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                          _("Thursday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[6] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                            _("Friday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[7] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                          _("Saturday: "), acol, arow, 1, 1);
    arow++;
    widgets->schedule_check_button[8] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                            _("Sunday: "), acol, arow, 1, 1);
    arow++;

    /* Hseparator */
    make_3columns_hseparator(GTK_GRID(widgets->settings_grid), acol, arow, 1, 1);
    arow++;

    /* Schedule hour   (0-23) */
    widgets->schedule_spin_button[1] =
        make_shortleft_spinbutton_with_label(GTK_GRID(widgets->settings_grid),
                                              _("Hour"), acol, arow, 1, 1, 20, 0, 23);
    arow++;

    /* Every hour   (*) */
    widgets->schedule_check_button[9] =
        make_checkbutton_with_label(GTK_GRID(widgets->settings_grid),
                                        _("Every hour: "), acol, arow, 1, 1);
    arow++;

    /* Hseparator */
    make_3columns_hseparator(GTK_GRID(widgets->settings_grid), acol, arow, 1, 1);
    arow++;

    /* Schedule minute (0-59) */
    widgets->schedule_spin_button[2] =
        make_shortleft_spinbutton_with_label(GTK_GRID(widgets->settings_grid),
                                            _("Minute"), acol, arow, 1, 1, 20, 0, 59);
    arow++;

    if(utf8 != NULL)
        g_free(utf8);

    gtk_widget_show_all(widgets->main_window);
}
